/*
         myDOG.h
 
 (c)2014 ARPix / Alexander Röhl
 
  
 CS0(CS) - pin 10  (SS pin)
 SDA(SI) - pin 11 (MOSI pin)
 CD(A0) - pin 9
 SCK(SCL) - pin 13 (SCK pin)
 RESET - pin RESET
 */

#include <Arduino.h>

#define DISPLAY_WIDTH 102
#define DISPLAY_HEIGHT 64
#define MAX_PAGE (DISPLAY_HEIGHT / 8)


class DOGClass  {
public:
  static void setChipSelectPin(const int);
  static void writeByte(unsigned int);
  static void Init();
  static void begin();
  static void drawPic(byte[8][102]);
  
  // (4) Set Column Address
  // Set the SRAM column address 
  // CA=0..131
  static void setColumnAdress(int);
#define LSB 0B00000000
#define MSB 0B00011000

  // (5) fillDisplay()
  // PC0: 0=Booster OFF; 1=Booster ON
  // PC1: 0=Regulator OFF; 1=Regulator ON
  // PC2: 0=Follower OFF; 1=Follower ON
  static void setPowerControl(int);  

  // (6) Set Scroll Line 
  // Set the display startline number SL=0..63
  static void setScrollLine(int);  

  // (7) Set Page Address
  // Set the SRAM page address PA=0..7
  static void setPageAdress(int);

  // (8) Set VLCD Resistor Ratio
  // Configure internal resistor ratio
  // PC=0..7
  static void setResistorRatio(int);

  // (9) Set Electronic Volume
  // Adjust contrast of LCD panel
  // PM=0..63
  static void setElectronicVolume(int);

  // (10) Set All Pixel On
  // C1=0: show SRAM content
  // C1=1: Set all SEG-Drivers to ON
  static void setAllPixelOn(int);

  // (11) Set Inverse Display
  // C0=0: show normal SRAM content
  // C0=1: show inverse SRAM content
  static void setInverseDisplay(int);

  // (13) Set SEG direction 
  // MX=0: normal SEG 0..131 
  // MX=1: mirror SEG 131..0
  static void setSEG(int);
#define frontView 0B10100000
#define backView 0B10100001

  // (14) Set COM direction
  // MY=0: normal COM 0..63
  // MY=1: mirror COM 63..0
  static void setCOM(int);
#define topView 0B11001000
#define bottomView 0B11000000


  // (17) Set LCD Bias Ratio
  // BR: 0=1/9; 1=1/7
  static void setBiasRatio(int);
  static void displayClear();

  ///////////////////////////////////////////////////
  // Text
  ///////////////////////////////////////////////////  
  static void print(char*);
  static void print(char*, int);
  static void print(char*, int, int);
  static void print(long, int, int);

  //////////////////////////////////////////////////////
  // Debugging Functions
  //////////////////////////////////////////////////////
  int freeRam (void);

private:
  static int Command(unsigned int);
  static void reset();

};

extern DOGClass myDOG;

////////////////////////////////////////
// DOGClass 
////////////////////////////////////////

#include "binary.h"
#include "mySPI.h"
#include "myfont.h"

DOGClass myDOG;
int CHIP_SELECT_PIN = 10;

void DOGClass::setChipSelectPin(const int val) {
   CHIP_SELECT_PIN = val;
}

void DOGClass::Init()
{
  delayMicroseconds(10);
  digitalWrite(CHIP_SELECT_PIN, 1); //dog_spi_disable_client();
  delayMicroseconds(10);
  digitalWrite(CHIP_SELECT_PIN, 0); // dog_spi_enable_client();
  delayMicroseconds(10);

  spi.send(0x040); 		// (6)Set Scroll Line to 0
  spi.send(0x0a1); 		// (13)Set SEG direction ADC set to reverse 
  spi.send(0x0c0); 		// (14)Set COM direction
  spi.send(0x0a6); 		/* display normal, bit val 0: LCD pixel off. */
  spi.send(0x0a2); 		/* LCD bias 1/9 */
  spi.send(0x02f); 		/* all power  control circuits on */
  spi.send(0x027); 		/* regulator, booster and follower */
  spi.send(0x081); 		/* set contrast */
  spi.send(0x00e); 		/* contrast value, EA default: 0x010, previous value for S102: 0x0e */
  spi.send(0x0fa); 		/* Set Temp compensation */

  spi.send(0x090); 		/* 0.11 deg/c WP Off WC Off*/
  spi.send(0x0a5); 		// (13)Set SEG direction 
  spi.send(0x0af); 		/* display on */

  spi.send(0x0a5); 		/* display all points, ST7565, UC1610 */
  delayMicroseconds(300);
  spi.send(0x0a4); 		/* normal display  */
  delayMicroseconds(300);

  digitalWrite(CHIP_SELECT_PIN, 1); // disable CS0 CS SS
}


void DOGClass::begin() 
{
  pinMode(CHIP_SELECT_PIN, OUTPUT);
  spi.Init();
  Init();
}

void DOGClass::reset() {
  //  Software Reset
  Command(0B11100010);
  setPageAdress(1);
}

//////////////////////////////////////////////////////
// Funktionen SPI
//////////////////////////////////////////////////////
int DOGClass::Command(unsigned int val) {
  digitalWrite(_CD,0);
  delayMicroseconds(10);
  digitalWrite(CHIP_SELECT_PIN, 1); //dog_spi_disable_client();
  delayMicroseconds(10);
  digitalWrite(CHIP_SELECT_PIN, 0); // dog_spi_enable_client();
  delayMicroseconds(10);
  spi.send(val); 
  digitalWrite(CHIP_SELECT_PIN, 1); // disable CS0 CS SS
}

void DOGClass::writeByte(unsigned int val) {
  delayMicroseconds(10);
  digitalWrite(_CD,1);
  digitalWrite(CHIP_SELECT_PIN, 0); // enable CS0 CS SS
  spi.send(val); 
  delayMicroseconds(10);
  digitalWrite(CHIP_SELECT_PIN, 1); // disable CS0 CS SS
  digitalWrite(_CD,0);
}

void DOGClass::displayClear() {
  myDOG.setColumnAdress(0);
  myDOG.setPageAdress(0);
  
  // clear display
  for (int aa=0; aa<MAX_PAGE; aa++) { // row 0 to 8
    setPageAdress(aa);
    setColumnAdress(0x0);
    for (int bb=0; bb<DISPLAY_WIDTH; bb++) { // column 0 to 101px
      writeByte(0x0);
    }
  }
  myDOG.setColumnAdress(0);
  myDOG.setPageAdress(0);
}



//////////////////////////////////////////////////////
// Funktionen Initialisierung
//////////////////////////////////////////////////////
void DOGClass::setColumnAdress(int val) {
  Command(0x00 | (val & 0x0F)); // LSB
  Command(0x10 | ((val>>4) & 0x0F));  // MSB
}
void DOGClass::setPowerControl(int val) {  Command(0B00101000 + val);}
void DOGClass::setScrollLine(int val) {  if (val > 64) val = 64;  Command(0B01000000 + val);}
void DOGClass::setPageAdress(int val) {  if (val > 8) val = 8;  Command(0xb0 + val);}
void DOGClass::setResistorRatio(int val) {  Command(0B00100000 + val);}
void DOGClass::setElectronicVolume(int val) {  if (val > 63) val = 63;  Command(0B10000001);  Command(0B00 + val);}
void DOGClass::setAllPixelOn(int val) {  Command(0B10100100 + val);}
void DOGClass::setInverseDisplay(int val) {  Command(0B10100110 + val);}
void DOGClass::setSEG(int val) {  Command(val); }
void DOGClass::setCOM(int val) {  Command(val); }
void DOGClass::setBiasRatio(int val) {  Command(0B10100010 + val);}


///////////////////////////////////////////////////
// Text  myfont.h
/////////////////////////////////////////////////// 
void DOGClass::print(char* myStr) {  myfont.print(myStr);  }
void DOGClass::print(char* myStr, int row) {  myfont.print(myStr, row);  }
void DOGClass::print(char* myStr, int x, int y) {  myfont.print(myStr, x, y);  }
void DOGClass::print(long myInt, int x, int y) {  myfont.print(myInt, x, y);  }


/////////////////////////////////////////////////////
// Graphic
/////////////////////////////////////////////////////
void DOGClass::drawPic(byte bitmap[8][102]) {
  myDOG.setColumnAdress(0); 
  myDOG.setPageAdress(0);

  byte BITMAP_ROW[102];

  for (int ROW=0; ROW<8; ROW++) {
    for (int COL=0; COL<102; COL++) {
      myDOG.writeByte(bitmap[ROW][COL]);
    }
    myDOG.setColumnAdress(0); 
    myDOG.setPageAdress(ROW+1);
  }
  myDOG.setColumnAdress(0); 
  myDOG.setPageAdress(0);
}


///////////////////////////////////////////////////
// Debugging Tools
/////////////////////////////////////////////////// 
int DOGClass::freeRam () {
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}

